// src/entity.ts
import { createId } from "@paralleldrive/cuid2";
import { camelCase, snakeCase } from "change-case";
import {
  sql
} from "kysely";
import { z } from "zod";
function hasTimestamps(name, tables) {
  const tableName = snakeCase(name);
  const table = tables.find((t) => t.name === tableName);
  if (!table) {
    throw new Error(`Table "${tableName}" not found`);
  }
  const hasCreatedAt = table.columns.some((c) => c.name === "created_at");
  const hasUpdatedAt = table.columns.some((c) => c.name === "updated_at");
  return [hasCreatedAt, hasUpdatedAt];
}
function zodFromDbSchema(name, tables) {
  const tableName = snakeCase(name);
  const table = tables.find((t) => t.name === tableName);
  if (!table) {
    throw new Error(`Table "${tableName}" not found`);
  }
  const schema = {};
  for (const column of table.columns) {
    const fieldName = camelCase(column.name);
    let zodType;
    switch (column.dataType.toLowerCase()) {
      // String types
      case "varchar":
      case "char":
      case "text":
      case "tinytext":
      case "mediumtext":
      case "longtext":
        zodType = z.string();
        break;
      // Number types
      case "int":
      case "integer":
      case "tinyint":
      case "smallint":
      case "mediumint":
      case "bigint":
      case "int2":
      case "int4":
      case "int8":
        zodType = z.number().int();
        break;
      case "decimal":
      case "numeric":
      case "float":
      case "double":
      case "real":
        zodType = z.number();
        break;
      // Boolean types
      case "boolean":
      case "bool":
        zodType = z.boolean();
        break;
      // Date types
      case "date":
      case "time":
      case "timestamp":
      case "datetime":
        zodType = z.date();
        break;
      // JSON types
      case "json":
      case "jsonb":
        zodType = z.unknown();
        break;
      // UUID type
      case "uuid":
        zodType = z.string().uuid();
        break;
      default:
        zodType = z.unknown();
    }
    if (column.isNullable || fieldName === "createdAt" || fieldName === "id") {
      zodType = zodType.optional();
    }
    schema[fieldName] = zodType;
  }
  return z.object(schema);
}
var Entity = class {
  constructor(db, name, tables, _zod, id = () => createId(), timestamp = () => Date.now()) {
    this.db = db;
    this.name = name;
    this.id = id;
    this.timestamp = timestamp;
    if (_zod) this.zod = _zod;
    else {
      this.zod = zodFromDbSchema(this.name, tables);
    }
    [this.createdAt, this.updatedAt] = hasTimestamps(this.name, tables);
  }
  zod;
  createdAt = false;
  updatedAt = false;
  // biome-ignore lint/complexity/noBannedTypes: <explanation>
  query() {
    return this.db.selectFrom(this.name);
  }
  async createMany(data) {
    const results = [];
    for (const item of data) {
      const inserted = await this.db.insertInto(this.name).values({
        id: item.id ?? this.id?.(),
        created_at: this.createdAt ? item.createdAt ?? this.timestamp?.() : void 0,
        updated_at: this.updatedAt ? item.updatedAt ?? this.timestamp?.() : void 0,
        ...item
        // biome-ignore lint/suspicious/noExplicitAny: <explanation>
      }).returningAll().executeTakeFirstOrThrow();
      results.push(inserted);
    }
    return results;
  }
  async create(data) {
    const inserted = await this.db.insertInto(this.name).values({
      id: data.id ?? this.id?.(),
      created_at: this.createdAt ? data.createdAt ?? this.timestamp?.() : void 0,
      updated_at: this.updatedAt ? data.updatedAt ?? this.timestamp?.() : void 0,
      ...data
      // biome-ignore lint/suspicious/noExplicitAny: <explanation>
    }).returningAll().executeTakeFirstOrThrow();
    return inserted;
  }
  async all(filter, pagination) {
    let query = this.db.selectFrom(this.name).selectAll();
    if (filter) {
      for (const [key, value] of Object.entries(filter)) {
        query = query.where(key, "=", value);
      }
    }
    if (pagination) {
      if (pagination.limit) {
        query = query.limit(pagination.limit);
      }
      if (pagination.offset) {
        query = query.offset(pagination.offset);
      }
    }
    return await query.execute();
  }
  async get(filter) {
    const result = await this.all(filter);
    if (result.length === 0) return void 0;
    return result[0];
  }
  async update(data) {
    const updated = await this.db.updateTable(this.name).set({
      updatedAt: this.updatedAt ? this.timestamp?.() : void 0,
      // biome-ignore lint/suspicious/noExplicitAny: <explanation>
      ...data,
      id: void 0
      // don't overwrite id
    }).where("id", "=", data.id).returningAll().executeTakeFirstOrThrow();
    return updated;
  }
  async updateMany(data) {
    const results = [];
    for (const item of data) {
      const updated = await this.db.updateTable(this.name).set({
        updatedAt: this.updatedAt ? this.timestamp?.() : void 0,
        // biome-ignore lint/suspicious/noExplicitAny: <explanation>
        ...item,
        id: void 0
        // don't overwrite id
      }).where("id", "=", item.id).returningAll().executeTakeFirstOrThrow();
      results.push(updated);
    }
    return results;
  }
  async delete(filter) {
    let query = this.db.deleteFrom(this.name);
    for (const [key, value] of Object.entries(filter)) {
      query = query.where(key, "=", value);
    }
    return await query.execute();
  }
};
async function store(db, opts) {
  const tables = await db.introspection.getTables();
  const entityStore = (key) => {
    const options = opts?.[key];
    return new Entity(
      db,
      key,
      tables,
      options?.zod,
      options?.id,
      options?.timestamp
    );
  };
  entityStore.db = db;
  entityStore.opts = opts;
  return entityStore;
}

// src/database.ts
import { join } from "node:path";
import consola from "consola";
import { Migrator, sql as sql2 } from "kysely";
import {
  DEFAULT_NUMERIC_PARSER,
  DialectManager,
  generate
} from "kysely-codegen";

// src/env.ts
function dev() {
  return process.env.NODE_ENV === "development";
}
function prod() {
  return process.env.NODE_ENV === "production";
}
function test() {
  return process.env.NODE_ENV === "test";
}

// src/database.ts
function migrate(db) {
  return async (...fns) => {
    const migrations = {};
    for (const [idx, fn] of fns.entries()) {
      migrations[idx] = { up: fn };
    }
    const migrator = new Migrator({
      db,
      provider: {
        async getMigrations() {
          return migrations;
        }
      },
      allowUnorderedMigrations: true
    });
    const result = await migrator.migrateToLatest();
    consola.debug("migrations result", result);
    consola.info("\u2713 migrations applied");
    const dialectManager = new DialectManager({
      numericParser: DEFAULT_NUMERIC_PARSER
    });
    if (!test()) {
      const dialect = dialectManager.getDialect("kysely-bun-sqlite");
      await generate({
        camelCase: true,
        db,
        dialect,
        print: false,
        typeOnlyImports: true,
        runtimeEnums: true,
        outFile: join(
          process.cwd(),
          "node_modules",
          "plainstack",
          "dist",
          "db.d.ts"
        )
      });
    }
    consola.info("\u2713 types generated");
    if (process.env.PS_MIGRATE) {
      console.info("PS_MIGRATE=1, exiting process...");
      process.exit(0);
    }
  };
}
async function rollback(db, fn) {
  const err = null;
  try {
    await sql2.raw("BEGIN").execute(db);
    await fn(db);
  } finally {
    await sql2.raw("ROLLBACK").execute(db);
  }
  if (err) throw err;
}

// src/job.ts
import path from "node:path";
import consola2 from "consola";
import { defineQueue, defineWorker } from "plainjob";
function job(opts) {
  return {
    name: opts.name,
    run: opts.run
  };
}
function schedule(opts) {
  return {
    name: path.parse(opts.name).name,
    cron: opts.cron,
    run: opts.run
  };
}
function queue(opts) {
  const logger = consola2;
  return defineQueue({ connection: opts.connection, logger });
}
async function work(queue2, jobs, schedules) {
  const log = consola2;
  if (!Object.values(jobs).length && !Object.values(schedules).length) {
    log.warn("started worker, but no jobs or schedules to run");
  }
  for (const schedule2 of Object.values(schedules)) {
    log.debug(`scheduling ${schedule2.name} to run at ${schedule2.cron}`);
    queue2.schedule(schedule2.name, { cron: schedule2.cron });
  }
  log.debug(`starting worker for jobs: ${Object.keys(jobs).join(", ")}`);
  if (Object.values(schedules).length) {
    log.debug(
      `starting worker for schedules: ${Object.keys(schedules).join(", ")}`
    );
  }
  const workables = [...Object.values(jobs), ...Object.values(schedules)];
  const workers = workables.map(
    (w) => defineWorker(w.name, w.run, { queue: queue2, logger: log })
  );
  await Promise.all(workers.map((worker) => worker.start()));
}
async function perform(queue2, job2, data) {
  const log = consola2;
  if (test()) {
    log.warn("NODE_ENV=test, performing immediately job in test mode");
    await job2.run({ data });
    return { id: "test" };
  }
  log.debug(`performing job ${job2.name}`);
  return queue2.add(job2.name, { data });
}

// src/middleware/cache-form-data.ts
import { createMiddleware } from "hono/factory";
var multipartRegex = /^multipart\/form-data(; boundary=[A-Za-z0-9'()+_,\-./:=?]+)?$/;
var urlencodedRegex = /^application\/x-www-form-urlencoded$/;
function cacheFormData() {
  return createMiddleware(async (c, next) => {
    const contentType = c.req.header("Content-Type");
    if (!contentType || !(multipartRegex.test(contentType) || urlencodedRegex.test(contentType))) {
      await next();
    } else {
      c.req.bodyCache.formData = await c.req.raw.clone().formData();
      await next();
    }
  });
}

// src/toast.tsx
import { html } from "hono/html";
import { Fragment, jsx, jsxs } from "hono/jsx/jsx-runtime";
function Toast(props) {
  if (!props.toast || typeof props.toast !== "string") {
    return null;
  }
  return /* @__PURE__ */ jsxs(Fragment, { children: [
    /* @__PURE__ */ jsx(
      "link",
      {
        rel: "stylesheet",
        type: "text/css",
        href: "https://cdn.jsdelivr.net/npm/toastify-js/src/toastify.min.css"
      }
    ),
    /* @__PURE__ */ jsx(
      "script",
      {
        type: "text/javascript",
        src: "https://cdn.jsdelivr.net/npm/toastify-js"
      }
    ),
    html`
        <script>
          window.onload = function() {
            Toastify({
              text: "${props.toast}",
              node: ${props.node ? JSON.stringify(props.node) : "undefined"},
              duration: ${props.duration || "3000"},
              selector: ${props.selector ? `"${props.selector}"` : "undefined"},
              destination: ${props.destination ? `"${props.destination}"` : "undefined"},
              newWindow: ${props.newWindow || "false"},
              close: ${props.close || "false"},
              gravity: "${props.gravity || "bottom"}",
              position: "${props.position || "right"}",
              ariaLive: "${props.ariaLive || "polite"}",
              backgroundColor: ${props.backgroundColor ? `"${props.backgroundColor}"` : "undefined"},
              avatar: ${props.avatar ? `"${props.avatar}"` : "undefined"},
              className: ${props.className ? `"${props.className}"` : "undefined"},
              stopOnFocus: ${props.stopOnFocus !== void 0 ? props.stopOnFocus : "true"},
              callback: ${props.callback ? `function() { (${props.callback.toString()})(); }` : "undefined"},
              onClick: ${props.onClick ? `function() { (${props.onClick.toString()})(); }` : "undefined"},
              offset: ${props.offset ? JSON.stringify(props.offset) : "undefined"},
              escapeMarkup: ${props.escapeMarkup !== void 0 ? props.escapeMarkup : "true"},
              style: ${props.style ? JSON.stringify(props.style) : "{}"},
              oldestFirst: ${props.oldestFirst || "false"}
            }).showToast();
          }
        </script>
      `
  ] });
}

// src/auth.ts
import { createMiddleware as createMiddleware2 } from "hono/factory";
function protect({
  signin: signin2,
  getUser
}) {
  return createMiddleware2(async (c, next) => {
    const userId = c.get("session")?.get("user-id");
    if (!userId) return c.redirect(signin2);
    const user = await getUser(userId);
    if (!user) return c.redirect(signin2);
    c.set("user", user);
    await next();
  });
}
function signin({
  getUser,
  protected: path2
}) {
  return createMiddleware2(async (c, next) => {
    const userId = c.get("session")?.get("user-id");
    const user = userId ? await getUser(userId) : void 0;
    if (user) return c.redirect("/protected");
    await next();
  });
}

// src/error.ts
import consola3 from "consola";
import { html as html2 } from "hono/html";
function error() {
  return (err, c) => {
    consola3.error(err);
    if (prod()) return c.text("Internal Server Error", 500);
    return c.html(
      html2`
      <!DOCTYPE html>
      <html lang="en">
        <head>
          <meta charset="UTF-8">
          <meta name="viewport" content="width=device-width, initial-scale=1.0">
          <title>Error</title>
          <style>
            body {
              font-family: Arial, sans-serif;
              line-height: 1.6;
              color: #333;
              max-width: 800px;
              margin: 0 auto;
              padding: 20px;
            }
            h1 {
              color: #e74c3c;
            }
            pre {
              background-color: #f8f8f8;
              border: 1px solid #ddd;
              border-radius: 4px;
              padding: 15px;
              overflow-x: auto;
            }
          </style>
        </head>
        <body>
          <p><strong>${err.message}</strong></p>
          <pre>${err.stack}</pre>
        </body>
      </html>
    `,
      500
    );
  };
}
export {
  Toast,
  cacheFormData,
  dev,
  error,
  job,
  migrate,
  perform,
  prod,
  protect,
  queue,
  rollback,
  schedule,
  signin,
  store,
  test,
  work
};
//# sourceMappingURL=plainstack.js.map